@@ -590,9 +590,9 @@ g_vfs_backend_add_auto_info (GVfsBackend *backend,
 }
 
 void
-g_vfs_backend_set_block_requests (GVfsBackend *backend)
+g_vfs_backend_set_block_requests (GVfsBackend *backend, gboolean value)
 {
-  backend->priv->block_requests = TRUE;
+  backend->priv->block_requests = value;
 }
 
 gboolean
@@ -793,7 +793,7 @@ typedef struct
   const gchar *message;
   const gchar *choices[3];
 
-  gboolean no_more_processes;
+  gboolean completed;
 
   GAsyncReadyCallback callback;
   gpointer            user_data;
@@ -802,7 +802,7 @@ typedef struct
 } UnmountWithOpData;
 
 static void
-complete_unmount_with_op (UnmountWithOpData *data)
+complete_unmount_with_op (UnmountWithOpData *data, gboolean no_more_processes)
 {
   gboolean ret;
   GSimpleAsyncResult *simple;
@@ -811,22 +811,33 @@ complete_unmount_with_op (UnmountWithOpData *data)
 
   ret = TRUE;
 
-  if (data->no_more_processes)
+  simple = g_simple_async_result_new (G_OBJECT (data->backend),
+                                      data->callback,
+                                      data->user_data,
+                                      NULL);
+
+  if (no_more_processes)
     {
       /* do nothing, e.g. return TRUE to signal we should unmount */
     }
+  else if (!data->ret)
+    {
+      /*  If the "show-processes" signal wasn't handled */
+      g_simple_async_result_set_error (simple, G_IO_ERROR, G_IO_ERROR_BUSY,
+                                       _("File system is busy"));
+      ret = FALSE;
+    }
   else
     {
       if (data->aborted || data->choice == 1)
         {
+          g_simple_async_result_set_error (simple, G_IO_ERROR, G_IO_ERROR_FAILED_HANDLED,
+                                           "GMountOperation aborted");
           ret = FALSE;
         }
     }
 
-  simple = g_simple_async_result_new (G_OBJECT (data->backend),
-                                      data->callback,
-                                      data->user_data,
-                                      NULL);
+  data->completed = TRUE;
   g_simple_async_result_set_op_res_gboolean (simple, ret);
   g_simple_async_result_complete (simple);
   g_object_unref (simple);
@@ -840,7 +851,7 @@ on_show_processes_reply (GMountSource  *mount_source,
   UnmountWithOpData *data = user_data;
 
   /* Do nothing if we've handled this already */
-  if (data->no_more_processes)
+  if (data->completed)
     return;
 
   data->ret = g_mount_source_show_processes_finish (mount_source,
@@ -848,7 +859,7 @@ on_show_processes_reply (GMountSource  *mount_source,
                                                     &data->aborted,
                                                     &data->choice);
 
-  complete_unmount_with_op (data);
+  complete_unmount_with_op (data, FALSE);
 }
 
 static gboolean
@@ -862,19 +873,17 @@ on_update_processes_timeout (gpointer user_data)
     {
       /* no more processes, abort mount op */
       g_mount_source_abort (data->mount_source);
-      data->no_more_processes = TRUE;
-      complete_unmount_with_op (data);
+      complete_unmount_with_op (data, TRUE);
     }
   else
     {
-      /* ignore reply */
       processes = g_vfs_daemon_get_blocking_processes (daemon);
       g_mount_source_show_processes_async (data->mount_source,
                                            data->message,
                                            processes,
                                            data->choices,
-                                           NULL,
-                                           NULL);
+                                           (GAsyncReadyCallback) on_show_processes_reply,
+                                           data);
       g_array_unref (processes);
     }
 
@@ -894,22 +903,28 @@ unmount_with_op_data_free (UnmountWithOpData *data)
  * @backend: A #GVfsBackend.
  * @res: A #GAsyncResult obtained from the @callback function passed
  *     to g_vfs_backend_unmount_with_operation().
+ * @error: A #GError, or NULL.
  *
  * Gets the result of the operation started by
  * gvfs_backend_unmount_with_operation_sync().
  *
+ * If the operation was cancelled, G_IO_ERROR_FAILED_HANDLED will be returned.
+ * If the operation wasn't interacted and there were outstanding jobs,
+ * G_IO_ERROR_BUSY will be returned.
+ *
  * Returns: %TRUE if the backend should be unmounted (either no blocking
  *     processes or the user decided to unmount anyway), %FALSE if
- *     no action should be taken.
+ *     no action should be taken (error is set).
  */
 gboolean
 g_vfs_backend_unmount_with_operation_finish (GVfsBackend *backend,
-                                             GAsyncResult *res)
+                                             GAsyncResult *res,
+                                             GError **error)
 {
   gboolean ret;
   GSimpleAsyncResult *simple = G_SIMPLE_ASYNC_RESULT (res);
 
-  if (g_simple_async_result_propagate_error (simple, NULL))
+  if (g_simple_async_result_propagate_error (simple, error))
     {
       ret = FALSE;
     }
@@ -1036,7 +1051,7 @@ forced_unregister_mount_callback (GVfsDBusMountTracker *proxy,
 void
 g_vfs_backend_force_unmount (GVfsBackend *backend)
 {
-  g_vfs_backend_set_block_requests (backend);
+  g_vfs_backend_set_block_requests (backend, TRUE);
   g_vfs_backend_unregister_mount (backend,
 				  (GAsyncReadyCallback) forced_unregister_mount_callback,
 				  backend);
